/*
OpenIO SDS metautils
Copyright (C) 2014 Worldine, original work as part of Redcurrant
Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 3.0 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.
*/

#ifndef OIO_SDS__metautils__lib__notifications_h
# define OIO_SDS__metautils__lib__notifications_h 1

#include <glib.h>
#include <metautils/lib/metautils.h>

// Kafka docs says that 10000 partitions is huge but should work
#define MAX_TOPIC_PARTITIONS 16384

struct metautils_notifier_s;
typedef struct metautils_notifier_s metautils_notifier_t;

/**
 * Allocate and initialize a notification handle.
 */
void metautils_notifier_init(metautils_notifier_t **notifier,
	const gchar *ns_name, struct grid_lbpool_s *lbpool);

/**
 * Clear a notification handle.
 */
void metautils_notifier_clear(metautils_notifier_t **notifier);

/**
 * Configure the notifier to send notifications to Kafka.
 *
 * @param notifier A notifier instance
 */
GError *metautils_notifier_init_kafka(metautils_notifier_t *notifier);

/**
 * Free the Kafka notifier only. Other notifiers will continue to work.
 */
void metautils_notifier_free_kafka(metautils_notifier_t *notifier);

/**
 * Create a Kafka topic instance, and add it to the cache.
 * This will speed up the notification process and avoid allocating a topic
 * on each call to metautils_notifier_send().
 */
GError *metautils_notifier_prepare_kafka_topic(metautils_notifier_t *notifier,
		 const gchar *topic_name);

/**
 * Send a raw notification to the specified topic.
 *
 * @param lb_key Pointer to a 32 bit integer helping for
 *   notification load balancing (can be NULL)
 */
GError *metautils_notifier_send_raw(metautils_notifier_t *notifier,
	const gchar *topic, GByteArray *data, const guint32 *lb_key);

/**
 * Send a JSON notification to the specifier topic. The template requires
 * a source address and a type, and includes an autogenerated sequence number.
 *
 * @param lb_key Pointer to a 32 bit integer helping for
 *   notification load balancing (can be NULL)
 */
GError *metautils_notifier_send_json(metautils_notifier_t *notifier,
	const gchar *topic, const gchar *src_addr, const char *notif_type,
	const gchar *notif_data, const guint32 *lb_key);

#endif /*OIO_SDS__metautils__lib__notifications_h*/